import React from 'react'

class UniversalCanvas extends React.Component {
  constructor(props) {
    super(props)
    this.state = {
      ratio: 1
    }
    this.onTouchStart = this.onTouchStart.bind(this)
    this.onTouchMove = this.onTouchMove.bind(this)
    this.onTouchEnd = this.onTouchEnd.bind(this)
    this.setup = this.setup.bind(this)
  }
  get width() {
    return this._canvas ? this._canvas.width : 0
  }
  get height() {
    return this._canvas ? this._canvas.height : 0
  }
  update() {
  }
  get ratio() {
    return this.state.ratio
  }
  getContext() {
    return this._canvas ? this._canvas.getContext('2d') : undefined
  }
  onTouchStart(evt) {
    let { onTouchStart } = this.props
    let { clientX, clientY } = evt.touches ? evt.touches[0] : evt
    this._mousedown = true
    if(!this._canvas) return
    if(evt.preventDefault) evt.preventDefault()
    if(onTouchStart) onTouchStart({
      clientX: clientX - this._canvas.offsetLeft,
      clientY: clientY - this._canvas.offsetTop
    })
  }
  onTouchMove(evt) {
    let { onHoverMove, onTouchMove } = this.props
    let { clientX, clientY } = evt.touches ? evt.touches[0] : evt
    if(!this._canvas) return
    if(evt.preventDefault) evt.preventDefault()
    if(this._mousedown) {
      if(onTouchMove) onTouchMove({
        clientX: clientX - this._canvas.offsetLeft,
        clientY: clientY - this._canvas.offsetTop
      })
    } else {
      if(onHoverMove) onHoverMove({
        clientX: clientX - this._canvas.offsetLeft,
        clientY: clientY - this._canvas.offsetTop
      })
    }
  }
  onTouchEnd(evt) {
    let { onTouchEnd } = this.props
    this._mousedown = false
    if(evt.preventDefault) evt.preventDefault()
    if(onTouchEnd) onTouchEnd({})
  }
  trigger(evt) {
    switch(evt.type) {
      case 'touchstart':
      case 'mousedown':
        this.onTouchStart(evt)
        break
      case 'hovermove':
      case 'touchmove':
      case 'mousemove':
        this.onTouchMove(evt)
        break
      case 'touchend':
      case 'mouseup':
        this.onTouchEnd(evt)
        break
    }
  }
  setup(canvas) {
    let { style, width, height } = this.props
    this._canvas = canvas
    let context = this.getContext()
    if(!context) return
    let devicePixelRatio = window.devicePixelRatio || 1
    let backingStoreRatio =
      context.webkitBackingStorePixelRatio ||
      context.mozBackingStorePixelRatio ||
      context.msBackingStorePixelRatio ||
      context.oBackingStorePixelRatio ||
      context.backingStorePixelRatio || 1
    let ratio = devicePixelRatio / backingStoreRatio;
    this.setState({
      ratio: ratio,
      fWidth: (width || style.width) * ratio,
      fHeight: (height || style.height) * ratio
    })
  }
  componentWillReceiveProps(nextProps) {
    let { style, width, height } = this.props
    let currentWidth = width || style.width
    let currentHeight = height || style.height
    let nextStyle = nextProps.style || {}
    let nextWidth = nextProps.width || nextStyle.width
    let nextHeight = nextProps.height || nextStyle.height
    if(currentWidth !== nextWidth || currentHeight !== nextHeight) {
      this.setup(this._canvas)
    }
  }
  render() {
    let { style, width, height } = this.props
    let { fWidth, fHeight } = this.state
    return (
      <canvas
        style={{
          ...style,
          width: `${width || style.width}px`,
          height: `${height || style.height}px`
        }}
        ref={this.setup}
        onTouchStart={this.onTouchStart}
        onTouchMove={this.onTouchMove}
        onTouchEnd={this.onTouchEnd}
        onMouseDown={this.onTouchStart}
        onMouseMove={this.onTouchMove}
        onMouseUp={this.onTouchEnd}
        width={fWidth || width || style.width}
        height={fHeight || height || style.height }
      />
    )
  }
}
UniversalCanvas.defaultProps = {
}

module.exports = UniversalCanvas
